@@ -2,84 +2,52 @@ pub mod cli_opts;
 pub mod model;
 pub mod read;
 
-use crate::config::{
-    model::{
-        DefinitiveHeloResults, EnhancedStatusCode, ExpModification, Header, HeaderType,
-        LogDestination, LogLevel, RejectResults, ReplyCode, SkipSenders, SyslogFacility,
-        TrustedNetworks,
+use crate::{
+    config::{
+        cli_opts::CliOptions,
+        model::{
+            DefinitiveHeloResults, EnhancedStatusCode, ExpModification, Header, HeaderType,
+            LogDestination, LogLevel, RejectResults, ReplyCode, SkipSenders, Socket,
+            SyslogFacility, TrustedNetworks,
+        },
+        read::ReadConfigError,
     },
-    read::ReadConfigError,
+    resolver::{DomainResolver, MockResolver, Resolver},
 };
 use log::{error, info, warn};
-use once_cell::sync::{Lazy, OnceCell};
+use once_cell::sync::Lazy;
 use std::{
     error::Error,
     fmt::{self, Display, Formatter},
     mem,
-    sync::{
-        atomic::{AtomicBool, Ordering},
-        Arc, Mutex, MutexGuard,
-    },
+    sync::{Arc, Mutex},
     time::Duration,
 };
-use viaspf::record::ExplainString;
+use viaspf_record::ExplainString;
 
-static CONFIG: OnceCell<Mutex<Arc<Config>>> = OnceCell::new();
-
-/// Initialises the static configuration `Mutex`.
-///
-/// Must be called exactly once during startup.
-pub fn init(config: Config) {
-    milter::set_debug_level(config.milter_debug_level());
-    CONFIG
-        .set(Mutex::new(Arc::new(config)))
-        .expect("configuration already initialized");
+/// A runtime configuration, containing immutable configuration and resolver.
+pub struct RuntimeConfig {
+    pub config: Config,
+    pub resolver: Resolver,
 }
 
-/// Returns an `Arc` with the current configuration.
-pub fn current() -> Arc<Config> {
-    let locked_config = lock_config();
-    Arc::clone(&locked_config)
-}
-
-fn reset_config(config: Config) -> Arc<Config> {
-    let mut locked_config = lock_config();
-    milter::set_debug_level(config.milter_debug_level());
-    mem::replace(&mut locked_config, Arc::new(config))
-}
-
-fn lock_config() -> MutexGuard<'static, Arc<Config>> {
-    CONFIG
-        .get()
-        .expect("configuration not initialized")
-        .lock()
-        .expect("could not get configuration lock")
-}
-
-static RELOAD_CONFIG: Lazy<Arc<AtomicBool>> = Lazy::new(|| Arc::new(AtomicBool::new(false)));
-
-/// Returns a reference to the static `Arc` containing the reload flag.
-pub fn get_reload_flag() -> &'static Arc<AtomicBool> {
-    &RELOAD_CONFIG
-}
-
-/// Sets the reload flag.
-pub fn set_reload() {
-    RELOAD_CONFIG.store(true, Ordering::Relaxed);
-}
-
-/// Reloads the configuration from the configuration file if the reload flag has
-/// been set.
-pub fn reload_if_stale() {
-    if !RELOAD_CONFIG.swap(false, Ordering::Relaxed) {
-        return;
+impl RuntimeConfig {
+    pub fn new(config: Config) -> Self {
+        let resolver = Resolver::Live(DomainResolver::new(config.timeout()));
+        Self { config, resolver }
     }
 
-    let opts = cli_opts::get();
+    pub fn with_mock_resolver(config: Config, resolver: MockResolver) -> Self {
+        let resolver = Resolver::Mock(Arc::new(resolver));
+        Self { config, resolver }
+    }
+}
 
+/// Reloads the configuration from the configuration file.
+pub async fn reload(current_runtime: &Mutex<Arc<RuntimeConfig>>, opts: &CliOptions) {
     let config_file = opts.config_file();
 
-    let config = match read::read_config(opts) {
+    let config = match read::read_config(opts).await {
         Ok(config) => config,
         Err(e) => {
             error!(
@@ -91,27 +59,39 @@ pub fn reload_if_stale() {
         }
     };
 
-    info!("configuration reloaded from {}", config_file.display());
-
     // Extract values of parameters that cannot be reloaded, reset the
     // configuration, then log a warning.
     let new_log_destination = config.log_destination();
     let new_log_level = config.log_level();
-    let new_socket = config.socket().to_owned();
+    let new_socket = config.socket().clone();
     let new_syslog_facility = config.syslog_facility();
 
-    let old_config = reset_config(config);
+    let old_runtime = {
+        let mut locked_runtime = current_runtime
+            .lock()
+            .expect("could not get configuration lock");
 
-    if new_log_destination != old_config.log_destination() {
+        let resolver = match &locked_runtime.resolver {
+            Resolver::Live(_) => Resolver::Live(DomainResolver::new(config.timeout())),
+            Resolver::Mock(m) => Resolver::Mock(m.clone()),
+        };
+        let runtime = RuntimeConfig { config, resolver };
+
+        mem::replace(&mut *locked_runtime, Arc::new(runtime))
+    };
+
+    info!("configuration reloaded from {}", config_file.display());
+
+    if new_log_destination != old_runtime.config.log_destination() {
         warn_changed_param("log_destination");
     }
-    if new_log_level != old_config.log_level() {
+    if new_log_level != old_runtime.config.log_level() {
         warn_changed_param("log_level");
     }
-    if new_socket != old_config.socket() {
+    if new_socket != *old_runtime.config.socket() {
         warn_changed_param("socket");
     }
-    if new_syslog_facility != old_config.syslog_facility() {
+    if new_syslog_facility != old_runtime.config.syslog_facility() {
         warn_changed_param("syslog_facility");
     }
 }
@@ -136,21 +116,21 @@ pub struct Config {
     include_mailfrom_local_part: bool,
     log_destination: LogDestination,
     log_level: LogLevel,
+    max_lookups: usize,
     max_void_lookups: usize,
-    milter_debug_level: i32,
     permerror_reply_code: ReplyCode,
-    permerror_reply_text: String,
+    permerror_reply_text: ExplainString,
     permerror_status_code: EnhancedStatusCode,
     reject_helo_results: RejectResults,
     reject_results: RejectResults,
     skip_senders: SkipSenders,
-    socket: String,
+    socket: Socket,
     softfail_reply_code: ReplyCode,
-    softfail_reply_text: String,
+    softfail_reply_text: ExplainString,
     softfail_status_code: EnhancedStatusCode,
     syslog_facility: SyslogFacility,
     temperror_reply_code: ReplyCode,
-    temperror_reply_text: String,
+    temperror_reply_text: ExplainString,
     temperror_status_code: EnhancedStatusCode,
     timeout: Duration,
     trust_authenticated_senders: bool,
@@ -159,7 +139,7 @@ pub struct Config {
 }
 
 impl Config {
-    pub fn builder<S: Into<String>>(socket: S) -> ConfigBuilder {
+    pub fn builder(socket: Socket) -> ConfigBuilder {
         ConfigBuilder::new(socket)
     }
 
@@ -219,19 +199,19 @@ impl Config {
         self.log_level
     }
 
-    pub fn max_void_lookups(&self) -> usize {
-        self.max_void_lookups
+    pub fn max_lookups(&self) -> usize {
+        self.max_lookups
     }
 
-    pub fn milter_debug_level(&self) -> i32 {
-        self.milter_debug_level
+    pub fn max_void_lookups(&self) -> usize {
+        self.max_void_lookups
     }
 
     pub fn permerror_reply_code(&self) -> &ReplyCode {
         &self.permerror_reply_code
     }
 
-    pub fn permerror_reply_text(&self) -> &str {
+    pub fn permerror_reply_text(&self) -> &ExplainString {
         &self.permerror_reply_text
     }
 
@@ -251,7 +231,7 @@ impl Config {
         &self.skip_senders
     }
 
-    pub fn socket(&self) -> &str {
+    pub fn socket(&self) -> &Socket {
         &self.socket
     }
 
@@ -259,7 +239,7 @@ impl Config {
         &self.softfail_reply_code
     }
 
-    pub fn softfail_reply_text(&self) -> &str {
+    pub fn softfail_reply_text(&self) -> &ExplainString {
         &self.softfail_reply_text
     }
 
@@ -275,7 +255,7 @@ impl Config {
         &self.temperror_reply_code
     }
 
-    pub fn temperror_reply_text(&self) -> &str {
+    pub fn temperror_reply_text(&self) -> &ExplainString {
         &self.temperror_reply_text
     }
 
@@ -341,8 +321,10 @@ static DEFAULT_FAIL_REPLY_TEXT: Lazy<ExplainString> =
     Lazy::new(|| DEFAULT_FAIL_REPLY_STRING.parse().unwrap());
 static DEFAULT_FAIL_REPLY_TEXT_EXP_PREFIX: Lazy<ExplainString> =
     Lazy::new(|| "SPF validation failed: %{o} explains: ".parse().unwrap());
+static DEFAULT_ERROR_REPLY_TEXT: Lazy<ExplainString> =
+    Lazy::new(|| DEFAULT_ERROR_REPLY_STRING.parse().unwrap());
 
-/// A builder for runtime configurations.
+/// A builder for configurations.
 ///
 /// This builder’s methods don’t validate inputs, instead all work is postponed
 /// until `build` is called. Instantiation of the builder does not instantiate
@@ -363,21 +345,21 @@ pub struct ConfigBuilder {
     include_mailfrom_local_part: Option<bool>,
     log_destination: Option<LogDestination>,
     log_level: Option<LogLevel>,
+    max_lookups: Option<usize>,
     max_void_lookups: Option<usize>,
-    milter_debug_level: Option<i32>,
     permerror_reply_code: Option<ReplyCode>,
-    permerror_reply_text: Option<String>,
+    permerror_reply_text: Option<ExplainString>,
     permerror_status_code: Option<EnhancedStatusCode>,
     reject_helo_results: Option<RejectResults>,
     reject_results: Option<RejectResults>,
     skip_senders: Option<SkipSenders>,
-    socket: String,
+    socket: Socket,
     softfail_reply_code: Option<ReplyCode>,
-    softfail_reply_text: Option<String>,
+    softfail_reply_text: Option<ExplainString>,
     softfail_status_code: Option<EnhancedStatusCode>,
     syslog_facility: Option<SyslogFacility>,
     temperror_reply_code: Option<ReplyCode>,
-    temperror_reply_text: Option<String>,
+    temperror_reply_text: Option<ExplainString>,
     temperror_status_code: Option<EnhancedStatusCode>,
     timeout: Option<Duration>,
     trust_authenticated_senders: Option<bool>,
@@ -388,7 +370,7 @@ pub struct ConfigBuilder {
 // The builder methods use `Into` with non-`Copy` type arguments, where a
 // sensible `From` implementation exists.
 impl ConfigBuilder {
-    pub fn new<S: Into<String>>(socket: S) -> Self {
+    pub fn new(socket: Socket) -> Self {
         Self {
             authserv_id: Default::default(),
             definitive_helo_results: Default::default(),
@@ -404,15 +386,15 @@ impl ConfigBuilder {
             include_mailfrom_local_part: Default::default(),
             log_destination: Default::default(),
             log_level: Default::default(),
+            max_lookups: Default::default(),
             max_void_lookups: Default::default(),
-            milter_debug_level: Default::default(),
             permerror_reply_code: Default::default(),
             permerror_reply_text: Default::default(),
             permerror_status_code: Default::default(),
             reject_helo_results: Default::default(),
             reject_results: Default::default(),
             skip_senders: Default::default(),
-            socket: socket.into(),
+            socket,
             softfail_reply_code: Default::default(),
             softfail_reply_text: Default::default(),
             softfail_status_code: Default::default(),
@@ -432,7 +414,7 @@ impl ConfigBuilder {
         self
     }
 
-    pub fn definitive_helo_results<R: Into<DefinitiveHeloResults>>(mut self, value: R) -> Self {
+    pub fn definitive_helo_results<T: Into<DefinitiveHeloResults>>(mut self, value: T) -> Self {
         self.definitive_helo_results = Some(value.into());
         self
     }
@@ -467,7 +449,7 @@ impl ConfigBuilder {
         self
     }
 
-    pub fn header<H: Into<Header>>(mut self, value: H) -> Self {
+    pub fn header<T: Into<Header>>(mut self, value: T) -> Self {
         self.header = Some(value.into());
         self
     }
@@ -497,13 +479,13 @@ impl ConfigBuilder {
         self
     }
 
-    pub fn max_void_lookups(mut self, value: usize) -> Self {
-        self.max_void_lookups = Some(value);
+    pub fn max_lookups(mut self, value: usize) -> Self {
+        self.max_lookups = Some(value);
         self
     }
 
-    pub fn milter_debug_level(mut self, value: i32) -> Self {
-        self.milter_debug_level = Some(value);
+    pub fn max_void_lookups(mut self, value: usize) -> Self {
+        self.max_void_lookups = Some(value);
         self
     }
 
@@ -512,8 +494,8 @@ impl ConfigBuilder {
         self
     }
 
-    pub fn permerror_reply_text<S: Into<String>>(mut self, value: S) -> Self {
-        self.permerror_reply_text = Some(value.into());
+    pub fn permerror_reply_text(mut self, value: ExplainString) -> Self {
+        self.permerror_reply_text = Some(value);
         self
     }
 
@@ -522,12 +504,12 @@ impl ConfigBuilder {
         self
     }
 
-    pub fn reject_helo_results<R: Into<RejectResults>>(mut self, value: R) -> Self {
+    pub fn reject_helo_results<T: Into<RejectResults>>(mut self, value: T) -> Self {
         self.reject_helo_results = Some(value.into());
         self
     }
 
-    pub fn reject_results<R: Into<RejectResults>>(mut self, value: R) -> Self {
+    pub fn reject_results<T: Into<RejectResults>>(mut self, value: T) -> Self {
         self.reject_results = Some(value.into());
         self
     }
@@ -542,8 +524,8 @@ impl ConfigBuilder {
         self
     }
 
-    pub fn softfail_reply_text<S: Into<String>>(mut self, value: S) -> Self {
-        self.softfail_reply_text = Some(value.into());
+    pub fn softfail_reply_text(mut self, value: ExplainString) -> Self {
+        self.softfail_reply_text = Some(value);
         self
     }
 
@@ -562,8 +544,8 @@ impl ConfigBuilder {
         self
     }
 
-    pub fn temperror_reply_text<S: Into<String>>(mut self, value: S) -> Self {
-        self.temperror_reply_text = Some(value.into());
+    pub fn temperror_reply_text(mut self, value: ExplainString) -> Self {
+        self.temperror_reply_text = Some(value);
         self
     }
 
@@ -593,9 +575,9 @@ impl ConfigBuilder {
     }
 
     pub fn build(self) -> Result<Config, ConfigError> {
-        let timeout = self.timeout.unwrap_or_else(|| Duration::from_secs(20));
-        let milter_debug_level = self.milter_debug_level.unwrap_or(0);
+        let max_lookups = self.max_lookups.unwrap_or(10);
         let max_void_lookups = self.max_void_lookups.unwrap_or(2);
+        let timeout = self.timeout.unwrap_or_else(|| Duration::from_secs(20));
         let trust_authenticated_senders = self.trust_authenticated_senders.unwrap_or(true);
         let verify_helo = self.verify_helo.unwrap_or(true);
 
@@ -603,7 +585,7 @@ impl ConfigBuilder {
         let delete_incoming_authentication_results = self
             .delete_incoming_authentication_results
             .unwrap_or_else(|| {
-                header.iter().any(|h| matches!(h, HeaderType::AuthenticationResults))
+                header.iter().any(|&h| h == HeaderType::AuthenticationResults)
             });
 
         let reject_results = self.reject_results.unwrap_or_default();
@@ -633,7 +615,7 @@ impl ConfigBuilder {
         )?;
         let softfail_reply_text = self
             .softfail_reply_text
-            .unwrap_or_else(|| DEFAULT_FAIL_REPLY_STRING.into());
+            .unwrap_or_else(|| DEFAULT_FAIL_REPLY_TEXT.clone());
 
         let (temperror_reply_code, temperror_status_code) = ensure_compatible(
             self.temperror_reply_code.unwrap_or_else(|| DEFAULT_TEMPERROR_REPLY_CODE.clone()),
@@ -642,7 +624,7 @@ impl ConfigBuilder {
         )?;
         let temperror_reply_text = self
             .temperror_reply_text
-            .unwrap_or_else(|| DEFAULT_ERROR_REPLY_STRING.into());
+            .unwrap_or_else(|| DEFAULT_ERROR_REPLY_TEXT.clone());
 
         let (permerror_reply_code, permerror_status_code) = ensure_compatible(
             self.permerror_reply_code.unwrap_or_else(|| DEFAULT_ERROR_REPLY_CODE.clone()),
@@ -651,7 +633,7 @@ impl ConfigBuilder {
         )?;
         let permerror_reply_text = self
             .permerror_reply_text
-            .unwrap_or_else(|| DEFAULT_ERROR_REPLY_STRING.into());
+            .unwrap_or_else(|| DEFAULT_ERROR_REPLY_TEXT.clone());
 
         Ok(Config {
             authserv_id: self.authserv_id,
@@ -668,8 +650,8 @@ impl ConfigBuilder {
             include_mailfrom_local_part: self.include_mailfrom_local_part.unwrap_or_default(),
             log_destination: self.log_destination.unwrap_or_default(),
             log_level: self.log_level.unwrap_or_default(),
+            max_lookups,
             max_void_lookups,
-            milter_debug_level,
             permerror_reply_code,
             permerror_reply_text,
             permerror_status_code,
@@ -700,7 +682,11 @@ fn ensure_compatible(
     if status_code.is_compatible_with(&reply_code) {
         Ok((reply_code, status_code))
     } else {
-        Err(ConfigError::IncompatibleStatusCodes(reply_code, status_code, result_kind.into()))
+        Err(ConfigError::IncompatibleStatusCodes(
+            reply_code,
+            status_code,
+            result_kind.into(),
+        ))
     }
 }
 
@@ -710,16 +696,16 @@ mod tests {
 
     #[test]
     fn delete_incoming_authentication_results_default_ok() {
-        let config = Config::builder("unused")
+        let config = Config::builder("unix:unused".parse().unwrap())
             .header(HeaderType::ReceivedSpf)
             .build()
             .unwrap();
-        assert_eq!(config.delete_incoming_authentication_results(), false);
+        assert!(!config.delete_incoming_authentication_results());
 
-        let config = Config::builder("unused")
+        let config = Config::builder("unix:unused".parse().unwrap())
             .header(HeaderType::AuthenticationResults)
             .build()
             .unwrap();
-        assert_eq!(config.delete_incoming_authentication_results(), true);
+        assert!(config.delete_incoming_authentication_results());
     }
 }
