@@ -8,7 +8,50 @@ use std::{
     str::FromStr,
 };
 use syslog::Facility;
-use viaspf::{record::ExplainString, Name, ParseNameError, SpfResult};
+use viaspf::{DomainName, ParseParamError, SpfResult};
+use viaspf_record::ExplainString;
+
+/// An error indicating that a socket could not be parsed.
+#[derive(Clone, Copy, Debug, Default, Eq, Hash, PartialEq)]
+pub struct ParseSocketError;
+
+impl Error for ParseSocketError {}
+
+impl Display for ParseSocketError {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        write!(f, "failed to parse socket")
+    }
+}
+
+/// A socket specification.
+#[derive(Clone, Debug, Eq, Hash, PartialEq)]
+pub enum Socket {
+    Inet(String),
+    Unix(String),
+}
+
+impl Display for Socket {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::Inet(s) => write!(f, "inet:{}", s),
+            Self::Unix(s) => write!(f, "unix:{}", s),
+        }
+    }
+}
+
+impl FromStr for Socket {
+    type Err = ParseSocketError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        if let Some(s) = s.strip_prefix("inet:") {
+            Ok(Self::Inet(s.into()))
+        } else if let Some(s) = s.strip_prefix("unix:") {
+            Ok(Self::Unix(s.into()))
+        } else {
+            Err(ParseSocketError)
+        }
+    }
+}
 
 #[derive(Clone, Copy, Debug, Default, Eq, Hash, PartialEq)]
 pub struct ParseStatusCodeError;
@@ -257,11 +300,12 @@ impl RejectResults {
 
 impl Default for RejectResults {
     fn default() -> Self {
-        let mut results = HashSet::new();
-        results.insert(RejectResultKind::Fail);
-        results.insert(RejectResultKind::Temperror);
-        results.insert(RejectResultKind::Permerror);
-        results.into()
+        HashSet::from([
+            RejectResultKind::Fail,
+            RejectResultKind::Temperror,
+            RejectResultKind::Permerror,
+        ])
+        .into()
     }
 }
 
@@ -357,15 +401,15 @@ impl Default for TrustedNetworks {
 #[derive(Clone, Debug, Eq, Hash, PartialEq)]
 pub struct SkipEntry {
     pub local_part: Option<String>,
-    pub domain: Name,
+    pub domain: DomainName,
     pub match_subdomains: bool,
 }
 
 impl SkipEntry {
     fn matches(&self, sender: &str) -> bool {
-        fn matches_domain(entry: &SkipEntry, domain: &Name) -> bool {
+        fn matches_domain(entry: &SkipEntry, domain: &DomainName) -> bool {
             if entry.match_subdomains {
-                domain.is_subdomain_of(&entry.domain)
+                domain.as_ref().is_subdomain_of(entry.domain.as_ref())
             } else {
                 domain == &entry.domain
             }
@@ -383,30 +427,30 @@ impl SkipEntry {
             }
         }
 
-        let (local_part, domain) = match sender.rfind('@') {
-            Some(i) => (Some(&sender[..i]), &sender[i + 1..]),
+        let (local_part, domain) = match sender.rsplit_once('@') {
+            Some((local_part, domain)) => (Some(local_part), domain),
             None => (None, sender),
         };
 
-        match Name::domain(domain) {
-            Ok(name) => matches_domain(self, &name) && matches_local_part(self, local_part),
+        match DomainName::new(domain) {
+            Ok(domain) => matches_domain(self, &domain) && matches_local_part(self, local_part),
             Err(_) => false,
         }
     }
 }
 
 impl FromStr for SkipEntry {
-    type Err = ParseNameError;
+    type Err = ParseParamError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         let mut local_part = None;
         let mut match_subdomains = false;
 
-        let domain = match s.rfind('@') {
-            Some(i) => {
+        let domain = match s.rsplit_once('@') {
+            Some((l, d)) => {
                 // Use local-part as-is without further validation.
-                local_part = Some(s[..i].into());
-                &s[i + 1..]
+                local_part = Some(l.into());
+                d
             }
             None => match s.strip_prefix('.') {
                 Some(s) => {
@@ -417,7 +461,7 @@ impl FromStr for SkipEntry {
             },
         };
 
-        let domain = Name::domain(domain)?;
+        let domain = DomainName::new(domain)?;
 
         Ok(Self {
             local_part,
@@ -527,12 +571,11 @@ impl FromStr for LogLevel {
 
 impl From<LogLevel> for LevelFilter {
     fn from(log_level: LogLevel) -> Self {
-        use LogLevel::*;
         match log_level {
-            Error => Self::Error,
-            Warn => Self::Warn,
-            Info => Self::Info,
-            Debug => Self::Debug,
+            LogLevel::Error => Self::Error,
+            LogLevel::Warn => Self::Warn,
+            LogLevel::Info => Self::Info,
+            LogLevel::Debug => Self::Debug,
         }
     }
 }
@@ -612,28 +655,27 @@ impl FromStr for SyslogFacility {
 
 impl From<SyslogFacility> for Facility {
     fn from(syslog_facility: SyslogFacility) -> Self {
-        use SyslogFacility::*;
         match syslog_facility {
-            Auth => Self::LOG_AUTH,
-            Authpriv => Self::LOG_AUTHPRIV,
-            Cron => Self::LOG_CRON,
-            Daemon => Self::LOG_DAEMON,
-            Ftp => Self::LOG_FTP,
-            Kern => Self::LOG_KERN,
-            Local0 => Self::LOG_LOCAL0,
-            Local1 => Self::LOG_LOCAL1,
-            Local2 => Self::LOG_LOCAL2,
-            Local3 => Self::LOG_LOCAL3,
-            Local4 => Self::LOG_LOCAL4,
-            Local5 => Self::LOG_LOCAL5,
-            Local6 => Self::LOG_LOCAL6,
-            Local7 => Self::LOG_LOCAL7,
-            Lpr => Self::LOG_LPR,
-            Mail => Self::LOG_MAIL,
-            News => Self::LOG_NEWS,
-            Syslog => Self::LOG_SYSLOG,
-            User => Self::LOG_USER,
-            Uucp => Self::LOG_UUCP,
+            SyslogFacility::Auth => Self::LOG_AUTH,
+            SyslogFacility::Authpriv => Self::LOG_AUTHPRIV,
+            SyslogFacility::Cron => Self::LOG_CRON,
+            SyslogFacility::Daemon => Self::LOG_DAEMON,
+            SyslogFacility::Ftp => Self::LOG_FTP,
+            SyslogFacility::Kern => Self::LOG_KERN,
+            SyslogFacility::Local0 => Self::LOG_LOCAL0,
+            SyslogFacility::Local1 => Self::LOG_LOCAL1,
+            SyslogFacility::Local2 => Self::LOG_LOCAL2,
+            SyslogFacility::Local3 => Self::LOG_LOCAL3,
+            SyslogFacility::Local4 => Self::LOG_LOCAL4,
+            SyslogFacility::Local5 => Self::LOG_LOCAL5,
+            SyslogFacility::Local6 => Self::LOG_LOCAL6,
+            SyslogFacility::Local7 => Self::LOG_LOCAL7,
+            SyslogFacility::Lpr => Self::LOG_LPR,
+            SyslogFacility::Mail => Self::LOG_MAIL,
+            SyslogFacility::News => Self::LOG_NEWS,
+            SyslogFacility::Syslog => Self::LOG_SYSLOG,
+            SyslogFacility::User => Self::LOG_USER,
+            SyslogFacility::Uucp => Self::LOG_UUCP,
         }
     }
 }
@@ -646,7 +688,7 @@ mod tests {
 
     #[test]
     fn reply_code_parse_ok() {
-        let code = "441".parse::<ReplyCode>();
+        let code = "441".parse();
         assert_eq!(code, Ok(ReplyCode::Transient("441".into())));
         let code = "499".parse::<ReplyCode>();
         assert_eq!(code, Err(ParseStatusCodeError));
@@ -654,9 +696,9 @@ mod tests {
 
     #[test]
     fn enhanced_status_code_parse_ok() {
-        let code = "4.1.23".parse::<EnhancedStatusCode>();
+        let code = "4.1.23".parse();
         assert_eq!(code, Ok(EnhancedStatusCode::Transient("4.1.23".into())));
-        let code = "4.0.23".parse::<EnhancedStatusCode>();
+        let code = "4.0.23".parse();
         assert_eq!(code, Ok(EnhancedStatusCode::Transient("4.0.23".into())));
         let code = "4.01.23".parse::<EnhancedStatusCode>();
         assert_eq!(code, Err(ParseStatusCodeError));
@@ -672,12 +714,9 @@ mod tests {
 
     #[test]
     fn trusted_networks_subnet_ok() {
+        let net = Ipv4Net::new([43, 5, 0, 0].into(), 16).unwrap();
         let trusted_networks = TrustedNetworks {
-            networks: {
-                let mut networks = HashSet::new();
-                networks.insert(Ipv4Net::new([43, 5, 0, 0].into(), 16).unwrap().into());
-                networks
-            },
+            networks: HashSet::from([net.into()]),
             ..Default::default()
         };
 
@@ -686,25 +725,23 @@ mod tests {
 
     #[test]
     fn skip_senders_ok() {
-        let skip_senders = SkipSenders::from({
-            let mut entries = HashSet::new();
-            entries.insert(SkipEntry {
+        let skip_senders = SkipSenders::from(HashSet::from([
+            SkipEntry {
                 local_part: None,
-                domain: Name::domain("example.com").unwrap(),
+                domain: DomainName::new("example.com").unwrap(),
                 match_subdomains: false,
-            });
-            entries.insert(SkipEntry {
+            },
+            SkipEntry {
                 local_part: None,
-                domain: Name::domain("super.example.com").unwrap(),
+                domain: DomainName::new("super.example.com").unwrap(),
                 match_subdomains: true,
-            });
-            entries.insert(SkipEntry {
+            },
+            SkipEntry {
                 local_part: Some("from".into()),
-                domain: Name::domain("example.org").unwrap(),
+                domain: DomainName::new("example.org").unwrap(),
                 match_subdomains: false,
-            });
-            entries
-        });
+            },
+        ]));
 
         assert!(skip_senders.includes("Example.Com"));
         assert!(skip_senders.includes("Me@Example.Com"));
