@@ -2,22 +2,22 @@ use crate::config::{
     cli_opts::CliOptions,
     model::{
         DefinitiveHeloResults, EnhancedStatusCode, ExpModification, Header, HeaderType,
-        LogDestination, LogLevel, RejectResults, ReplyCode, SkipSenders, SyslogFacility,
+        LogDestination, LogLevel, RejectResults, ReplyCode, SkipSenders, Socket, SyslogFacility,
         TrustedNetworks,
     },
     Config, ConfigBuilder, ConfigError,
 };
 use std::{
     collections::HashSet,
-    convert::TryInto,
     error::Error,
     fmt::{self, Display, Formatter},
-    fs, io,
+    io,
     net::IpAddr,
     path::{Path, PathBuf},
     time::Duration,
 };
-use viaspf::record::ExplainString;
+use tokio::fs;
+use viaspf_record::ExplainString;
 
 // Implementation note: Remember not to use variable-sized types such as `usize`
 // in `RawConfig`. For this reason and others, the raw configuration does not
@@ -41,22 +41,22 @@ struct RawConfig {
     include_mailfrom_local_part: Option<bool>,
     log_destination: Option<LogDestination>,
     log_level: Option<LogLevel>,
+    max_lookups: Option<u32>,
     max_void_lookups: Option<u32>,
-    milter_debug_level: Option<i32>,
     permerror_reply_code: Option<ReplyCode>,
-    permerror_reply_text: Option<String>,
+    permerror_reply_text: Option<ExplainString>,
     permerror_status_code: Option<EnhancedStatusCode>,
     reject_helo_results: Option<RejectResults>,
     reject_results: Option<RejectResults>,
     skip_senders: Option<SkipSenders>,
     skip_senders_file: Option<SkipSenders>,
-    socket: Option<String>,
+    socket: Option<Socket>,
     softfail_reply_code: Option<ReplyCode>,
-    softfail_reply_text: Option<String>,
+    softfail_reply_text: Option<ExplainString>,
     softfail_status_code: Option<EnhancedStatusCode>,
     syslog_facility: Option<SyslogFacility>,
     temperror_reply_code: Option<ReplyCode>,
-    temperror_reply_text: Option<String>,
+    temperror_reply_text: Option<ExplainString>,
     temperror_status_code: Option<EnhancedStatusCode>,
     timeout_secs: Option<u32>,
     trust_authenticated_senders: Option<bool>,
@@ -73,7 +73,7 @@ impl RawConfig {
         // Note: Keep string `socket` in sync with parsing and doc.
         let socket = cli_opts
             .socket()
-            .map(ToOwned::to_owned)
+            .cloned()
             .or(self.socket)
             .ok_or_else(|| ConfigError::MissingMandatoryParam("socket".into()))?;
 
@@ -121,6 +121,13 @@ impl RawConfig {
         if let Some(v) = self.log_level {
             builder = builder.log_level(v);
         }
+        if let Some(v) = self.max_lookups {
+            // Note: Keep string `max_lookups` in sync with parsing and doc.
+            builder = builder.max_lookups(
+                v.try_into()
+                    .map_err(|_| ConfigError::TypeConversion("max_lookups".into()))?,
+            );
+        }
         if let Some(v) = self.max_void_lookups {
             // Note: Keep string `max_void_lookups` in sync with parsing and doc.
             builder = builder.max_void_lookups(
@@ -128,9 +135,6 @@ impl RawConfig {
                     .map_err(|_| ConfigError::TypeConversion("max_void_lookups".into()))?,
             );
         }
-        if let Some(v) = self.milter_debug_level {
-            builder = builder.milter_debug_level(v);
-        }
         if let Some(v) = self.permerror_reply_code {
             builder = builder.permerror_reply_code(v);
         }
@@ -263,6 +267,7 @@ pub enum ParseParamError {
     UnknownKey(String),
     DuplicateKey(String),
     InvalidValue,
+    InvalidSocket(String),
     InvalidLogDestination(String),
     InvalidLogLevel(String),
     InvalidSyslogFacility(String),
@@ -277,7 +282,7 @@ pub enum ParseParamError {
     InvalidDefinitiveHeloResult(String),
     InvalidHeaderField(String),
     DuplicateHeaderField(HeaderType),
-    ReadIncludedFile(PathBuf, String),
+    ReadIncludedFile(PathBuf, String),  // I/O error recorded as string
     InvalidSkipEntry(String),
     InvalidSkipEntryInFile(String, PathBuf),
 }
@@ -291,6 +296,7 @@ impl Display for ParseParamError {
             Self::UnknownKey(key) => write!(f, "unknown parameter \"{}\"", key),
             Self::DuplicateKey(key) => write!(f, "duplicate parameter \"{}\"", key),
             Self::InvalidValue => write!(f, "invalid parameter value syntax"),
+            Self::InvalidSocket(s) => write!(f, "invalid socket \"{}\"", s),
             Self::InvalidLogDestination(s) => write!(f, "invalid log destination \"{}\"", s),
             Self::InvalidLogLevel(s) => write!(f, "invalid log level \"{}\"", s),
             Self::InvalidSyslogFacility(s) => write!(f, "invalid syslog facility \"{}\"", s),
@@ -324,21 +330,23 @@ pub fn focus_error(error: &ConfigError) -> &dyn Error {
     }
 }
 
-pub fn read_config(opts: &CliOptions) -> Result<Config, ConfigError> {
+pub async fn read_config(opts: &CliOptions) -> Result<Config, ConfigError> {
     let config_file = opts.config_file();
 
-    let raw_config = read_raw_config(config_file)?;
+    let raw_config = read_raw_config(config_file).await?;
 
     raw_config.into_builder(opts)?.build()
 }
 
-fn read_raw_config<P: AsRef<Path>>(path: P) -> Result<RawConfig, ReadConfigError> {
-    let s = fs::read_to_string(path)?;
+async fn read_raw_config<P: AsRef<Path>>(path: P) -> Result<RawConfig, ReadConfigError> {
+    let s = fs::read_to_string(path).await?;
 
-    parse_raw_config(&s).map_err(From::from)
+    let raw_config = parse_raw_config(&s).await?;
+
+    Ok(raw_config)
 }
 
-fn parse_raw_config(s: &str) -> Result<RawConfig, ParseConfigError> {
+async fn parse_raw_config(s: &str) -> Result<RawConfig, ParseConfigError> {
     let mut config = Default::default();
     let mut keys_seen = HashSet::new();
 
@@ -350,9 +358,8 @@ fn parse_raw_config(s: &str) -> Result<RawConfig, ParseConfigError> {
             continue;
         }
 
-        let mut parts = line.splitn(2, '=');
-        match (parts.next(), parts.next()) {
-            (Some(k), Some(v)) => {
+        match line.split_once('=') {
+            Some((k, v)) => {
                 let k = k.trim();
                 let v = v.trim();
 
@@ -364,11 +371,12 @@ fn parse_raw_config(s: &str) -> Result<RawConfig, ParseConfigError> {
                 }
 
                 parse_config_entry(&mut config, k, v)
+                    .await
                     .map_err(|e| ParseConfigError { line: num, kind: e })?;
 
                 keys_seen.insert(k);
             }
-            _ => {
+            None => {
                 return Err(ParseConfigError {
                     line: num,
                     kind: ParseParamError::InvalidLine,
@@ -382,7 +390,11 @@ fn parse_raw_config(s: &str) -> Result<RawConfig, ParseConfigError> {
 
 type ParseParamResult<T> = Result<T, ParseParamError>;
 
-fn parse_config_entry(config: &mut RawConfig, key: &str, value: &str) -> ParseParamResult<()> {
+async fn parse_config_entry(
+    config: &mut RawConfig,
+    key: &str,
+    value: &str,
+) -> ParseParamResult<()> {
     match key {
         "authserv_id" => config.authserv_id = Some(value.to_owned()),
         "definitive_helo_results" => config.definitive_helo_results = Some(value.parse_param()?),
@@ -398,22 +410,22 @@ fn parse_config_entry(config: &mut RawConfig, key: &str, value: &str) -> ParsePa
         "include_mailfrom_local_part" => config.include_mailfrom_local_part = Some(value.parse_param()?),
         "log_destination" => config.log_destination = Some(value.parse_param()?),
         "log_level" => config.log_level = Some(value.parse_param()?),
+        "max_lookups" => config.max_lookups = Some(value.parse_param()?),
         "max_void_lookups" => config.max_void_lookups = Some(value.parse_param()?),
-        "milter_debug_level" => config.milter_debug_level = Some(value.parse_param()?),
         "permerror_reply_code" => config.permerror_reply_code = Some(value.parse_param()?),
-        "permerror_reply_text" => config.permerror_reply_text = Some(value.to_owned()),
+        "permerror_reply_text" => config.permerror_reply_text = Some(value.parse_param()?),
         "permerror_status_code" => config.permerror_status_code = Some(value.parse_param()?),
         "reject_helo_results" => config.reject_helo_results = Some(value.parse_param()?),
         "reject_results" => config.reject_results = Some(value.parse_param()?),
         "skip_senders" => config.skip_senders = Some(value.parse_param()?),
-        "skip_senders_file" => config.skip_senders_file = Some(parse_skip_senders_file(value)?),
-        "socket" => config.socket = Some(value.to_owned()),
+        "skip_senders_file" => config.skip_senders_file = Some(parse_skip_senders_file(value).await?),
+        "socket" => config.socket = Some(value.parse_param()?),
         "softfail_reply_code" => config.softfail_reply_code = Some(value.parse_param()?),
-        "softfail_reply_text" => config.softfail_reply_text = Some(value.to_owned()),
+        "softfail_reply_text" => config.softfail_reply_text = Some(value.parse_param()?),
         "softfail_status_code" => config.softfail_status_code = Some(value.parse_param()?),
         "syslog_facility" => config.syslog_facility = Some(value.parse_param()?),
         "temperror_reply_code" => config.temperror_reply_code = Some(value.parse_param()?),
-        "temperror_reply_text" => config.temperror_reply_text = Some(value.to_owned()),
+        "temperror_reply_text" => config.temperror_reply_text = Some(value.parse_param()?),
         "temperror_status_code" => config.temperror_status_code = Some(value.parse_param()?),
         "timeout_secs" => config.timeout_secs = Some(value.parse_param()?),
         "trust_authenticated_senders" => config.trust_authenticated_senders = Some(value.parse_param()?),
@@ -462,6 +474,12 @@ impl FromParamStr for bool {
     }
 }
 
+impl FromParamStr for Socket {
+    fn from_param_str(s: &str) -> ParseParamResult<Self> {
+        s.parse().map_err(|_| ParseParamError::InvalidSocket(s.into()))
+    }
+}
+
 impl FromParamStr for ReplyCode {
     fn from_param_str(s: &str) -> ParseParamResult<Self> {
         s.parse().map_err(|_| ParseParamError::InvalidReplyCode(s.into()))
@@ -499,22 +517,22 @@ impl FromParamStr for Header {
 
 impl FromParamStr for ExpModification {
     fn from_param_str(s: &str) -> ParseParamResult<Self> {
-        let mut parts = s.splitn(2, "%{exp}");
-        match (parts.next(), parts.next()) {
-            (Some(es), None) => {
-                Ok(ExpModification::Substitute(es.parse().map_err(|_| {
-                    ParseParamError::InvalidExplainString(s.into())
-                })?))
+        match s.split_once("%{exp}") {
+            Some((prefix, suffix)) => {
+                let prefix = prefix
+                    .parse()
+                    .map_err(|_| ParseParamError::InvalidExplainString(s.into()))?;
+                let suffix = suffix
+                    .parse()
+                    .map_err(|_| ParseParamError::InvalidExplainString(s.into()))?;
+                Ok(Self::Decorate { prefix, suffix })
             }
-            (Some(prefix), Some(suffix)) => Ok(ExpModification::Decorate {
-                prefix: prefix
+            None => {
+                let es = s
                     .parse()
-                    .map_err(|_| ParseParamError::InvalidExplainString(s.into()))?,
-                suffix: suffix
-                    .parse()
-                    .map_err(|_| ParseParamError::InvalidExplainString(s.into()))?,
-            }),
-            _ => unreachable!(),
+                    .map_err(|_| ParseParamError::InvalidExplainString(s.into()))?;
+                Ok(Self::Substitute(es))
+            }
         }
     }
 }
@@ -601,8 +619,9 @@ impl FromParamStr for SkipSenders {
     }
 }
 
-fn parse_skip_senders_file(s: &str) -> ParseParamResult<SkipSenders> {
+async fn parse_skip_senders_file(s: &str) -> ParseParamResult<SkipSenders> {
     let file_content = fs::read_to_string(s)
+        .await
         .map_err(|e| ParseParamError::ReadIncludedFile(s.into(), e.to_string()))?;
 
     parse_skip_senders_file_content(&file_content, s)
@@ -658,67 +677,66 @@ fn is_ignored_line(line: &str) -> bool {
 mod tests {
     use super::*;
     use crate::config::model::{DefinitiveHeloResultKind, RejectResultKind, SkipEntry};
-    use viaspf::Name;
+    use viaspf::DomainName;
 
     #[test]
     fn socket_with_cli_options_override() {
         let config = RawConfig {
-            socket: Some("raw_config".into()),
+            socket: Some(Socket::Unix("raw_config".into())),
             ..Default::default()
         };
-        let opts = CliOptions::builder().socket("cli_opts").build();
+        let opts = CliOptions::builder()
+            .socket(Socket::Unix("cli_opts".into()))
+            .build();
 
         let config = config.into_builder(&opts).unwrap().build().unwrap();
-        assert_eq!(config.socket(), "cli_opts");
+        assert_eq!(config.socket(), &Socket::Unix("cli_opts".into()));
     }
 
-    #[test]
-    fn parse_results_ok() {
+    #[tokio::test]
+    async fn parse_results_ok() {
         let config = "
         definitive_helo_results = pass, fail, temperror, permerror
         reject_helo_results = fail, temperror
         reject_results = fail, softfail, permerror
         ";
 
-        let raw_config = parse_raw_config(config).unwrap();
+        let raw_config = parse_raw_config(config).await.unwrap();
 
         assert_eq!(
             raw_config.definitive_helo_results,
-            Some({
-                let mut results = HashSet::new();
-                results.insert(DefinitiveHeloResultKind::Pass);
-                results.insert(DefinitiveHeloResultKind::Fail);
-                results.insert(DefinitiveHeloResultKind::Temperror);
-                results.insert(DefinitiveHeloResultKind::Permerror);
-                results.into()
-            })
+            Some(
+                HashSet::from([
+                    DefinitiveHeloResultKind::Pass,
+                    DefinitiveHeloResultKind::Fail,
+                    DefinitiveHeloResultKind::Temperror,
+                    DefinitiveHeloResultKind::Permerror,
+                ])
+                .into()
+            )
         );
         assert_eq!(
             raw_config.reject_helo_results,
-            Some({
-                let mut results = HashSet::new();
-                results.insert(RejectResultKind::Fail);
-                results.insert(RejectResultKind::Temperror);
-                results.into()
-            })
+            Some(HashSet::from([RejectResultKind::Fail, RejectResultKind::Temperror]).into())
         );
         assert_eq!(
             raw_config.reject_results,
-            Some({
-                let mut results = HashSet::new();
-                results.insert(RejectResultKind::Fail);
-                results.insert(RejectResultKind::Softfail);
-                results.insert(RejectResultKind::Permerror);
-                results.into()
-            })
+            Some(
+                HashSet::from([
+                    RejectResultKind::Fail,
+                    RejectResultKind::Softfail,
+                    RejectResultKind::Permerror,
+                ])
+                .into()
+            )
         );
     }
 
-    #[test]
-    fn parse_results_invalid_result() {
+    #[tokio::test]
+    async fn parse_results_invalid_result() {
         let config = "reject_results = softfail, Failure";
 
-        let raw_config = parse_raw_config(config);
+        let raw_config = parse_raw_config(config).await;
 
         assert_eq!(
             raw_config,
@@ -752,23 +770,25 @@ mod tests {
 
     #[test]
     fn parse_skip_senders_ok() {
-        let skip_senders = "Example.Com, me@Example.Com".parse_param::<SkipSenders>().unwrap();
+        let skip_senders = "Example.Com, me@Example.Com"
+            .parse_param::<SkipSenders>()
+            .unwrap();
+
         assert_eq!(
             skip_senders,
-            SkipSenders::from({
-                let mut results = HashSet::new();
-                results.insert(SkipEntry {
+            HashSet::from([
+                SkipEntry {
                     local_part: None,
-                    domain: Name::domain("example.com").unwrap(),
+                    domain: DomainName::new("example.com").unwrap(),
                     match_subdomains: false,
-                });
-                results.insert(SkipEntry {
+                },
+                SkipEntry {
                     local_part: Some("me".into()),
-                    domain: Name::domain("example.com").unwrap(),
+                    domain: DomainName::new("example.com").unwrap(),
                     match_subdomains: false,
-                });
-                results
-            })
+                },
+            ])
+            .into()
         );
     }
 
@@ -787,38 +807,37 @@ mod tests {
 
         assert_eq!(
             skip_senders,
-            SkipSenders::from({
-                let mut results = HashSet::new();
-                results.insert(SkipEntry {
+            HashSet::from([
+                SkipEntry {
                     local_part: None,
-                    domain: Name::domain("example.com").unwrap(),
+                    domain: DomainName::new("example.com").unwrap(),
                     match_subdomains: false,
-                });
-                results.insert(SkipEntry {
+                },
+                SkipEntry {
                     local_part: None,
-                    domain: Name::domain("example.com").unwrap(),
+                    domain: DomainName::new("example.com").unwrap(),
                     match_subdomains: true,
-                });
-                results.insert(SkipEntry {
+                },
+                SkipEntry {
                     local_part: Some("me".into()),
-                    domain: Name::domain("example.com").unwrap(),
+                    domain: DomainName::new("example.com").unwrap(),
                     match_subdomains: false,
-                });
-                results
-            })
+                },
+            ])
+            .into()
         );
     }
 
     #[test]
     fn split_param_values_ok() {
-        assert!(split_param_values(" ").eq(vec![]));
-        assert!(split_param_values(" a").eq(vec![Ok("a")]));
-        assert!(split_param_values(" a,").eq(vec![Ok("a"), Err(ParseParamError::InvalidValue)]));
-        assert!(split_param_values(" ,a").eq(vec![Err(ParseParamError::InvalidValue), Ok("a")]));
-        assert!(split_param_values(" ,").eq(vec![
+        assert!(split_param_values(" ").eq([]));
+        assert!(split_param_values(" a").eq([Ok("a")]));
+        assert!(split_param_values(" a,").eq([Ok("a"), Err(ParseParamError::InvalidValue)]));
+        assert!(split_param_values(" ,a").eq([Err(ParseParamError::InvalidValue), Ok("a")]));
+        assert!(split_param_values(" ,").eq([
             Err(ParseParamError::InvalidValue),
             Err(ParseParamError::InvalidValue)
         ]));
-        assert!(split_param_values(" a,b").eq(vec![Ok("a"), Ok("b")]));
+        assert!(split_param_values(" a,b").eq([Ok("a"), Ok("b")]));
     }
 }
