@@ -1,13 +1,13 @@
 use crate::{
     auth::SpfResultKind,
     header::{format, HeaderField, HEADER_LINE_WIDTH},
-    verify::{self, Identity, VerificationResult},
+    verify::{Identity, VerificationResult},
 };
 use std::{
     borrow::Cow,
     fmt::{self, Display, Formatter},
 };
-use viaspf::SpfResultCause;
+use viaspf::{DomainName, SpfResultCause};
 
 // The `Authentication-Results` header field is defined in RFC 8601.
 
@@ -37,8 +37,8 @@ pub fn extract_authserv_id(value: &str) -> Option<Cow<'_, str>> {
     None
 }
 
-/// An `Authentication-Results` header, containing the header field content in
-/// an encoded, pre-formatted form.
+/// An `Authentication-Results` header, containing the header field body in an
+/// encoded, pre-formatted form.
 #[derive(Clone, Debug, Eq, Hash, PartialEq)]
 pub struct AuthenticationResultsHeader {
     body_pieces: Vec<String>,
@@ -112,16 +112,32 @@ fn make_auth_results_parts(
     parts
 }
 
-fn encode_sender(identity: &Identity, include_mailfrom_local_part: bool) -> Cow<'_, str> {
+fn encode_sender(identity: &Identity, include_mailfrom_local_part: bool) -> String {
     let mut sender = identity.as_ref();
 
     // The `Authentication-Results` header only records ‘authenticated’ data.
     // According to RFC 8601, even the MAIL FROM identity *should* therefore (by
     // default) only record the domain without the local-part.
-    if let Identity::MailFrom(s) = identity {
-        if !include_mailfrom_local_part {
-            if let Some(i) = s.rfind('@') {
-                sender = &s[i + 1..];
+    //
+    // Also, RFC 8616, section 5 states that internationalised domain names
+    // should be in Unicode form. See the reference to RFC 6376, section 3.5 and
+    // RFC 8601, section 2.2.
+
+    let s;
+    if let Identity::MailFrom(mail_from) = identity {
+        if let Some((l, d)) = mail_from.rsplit_once('@') {
+            let (udomain, e) = idna::domain_to_unicode(d);
+
+            // Only use flawless conversions, and only if something changed.
+            if e.is_ok() && !udomain.eq_ignore_ascii_case(d) {
+                if include_mailfrom_local_part {
+                    s = format!("{}@{}", l, udomain);
+                } else {
+                    s = udomain;
+                }
+                sender = &s;
+            } else if !include_mailfrom_local_part {
+                sender = d;
             }
         }
     };
@@ -132,7 +148,7 @@ fn encode_sender(identity: &Identity, include_mailfrom_local_part: bool) -> Cow<
     if is_valid_unquoted_pvalue(sender) {
         sender.into()
     } else {
-        format::encode_mime_value(sender)
+        format::encode_mime_value(sender).into()
     }
 }
 
@@ -143,14 +159,14 @@ fn is_valid_unquoted_pvalue(mut s: &str) -> bool {
     }
     // See RFC 6376, section 3.5, and RFC 8616.
     fn is_domain_name(s: &str) -> bool {
-        verify::is_fqdn(s)
+        DomainName::new(s).is_ok()
     }
 
-    if let Some(i) = s.rfind('@') {
-        if !is_local_part(&s[..i]) {
+    if let Some((l, d)) = s.rsplit_once('@') {
+        if !is_local_part(l) {
             return false;
         }
-        s = &s[i + 1..];
+        s = d;
     }
     is_domain_name(s)
 }
@@ -185,10 +201,8 @@ impl HeaderField for AuthenticationResultsHeader {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use viaspf::{
-        record::{Mechanism, A},
-        ErrorCause, SpfResult,
-    };
+    use viaspf::{ErrorCause, SpfResult};
+    use viaspf_record::{Mechanism, A};
 
     #[test]
     fn extract_authserv_id_ok() {
@@ -225,6 +239,23 @@ mod tests {
         assert_eq!(encode_sender(&MailFrom("\"me\"@例子.cn".into()), true), "\"me\"@例子.cn");
         assert_eq!(encode_sender(&MailFrom("me@例子".into()), true), "\"me@例子\"");
         assert_eq!(encode_sender(&MailFrom("me.@例子.cn".into()), true), "\"me.@例子.cn\"");
+
+        assert_eq!(
+            encode_sender(&Helo("mail.xn--fsqu00a.xn--fiqs8s".into()), false),
+            "mail.xn--fsqu00a.xn--fiqs8s"
+        );
+        assert_eq!(
+            encode_sender(&MailFrom("me@mail.xn--fsqu00a.xn--fiqs8s".into()), false),
+            "mail.例子.中国"
+        );
+        assert_eq!(
+            encode_sender(&MailFrom("me@mail.xn--fsqu00a.xn--fiqs8s".into()), true),
+            "me@mail.例子.中国"
+        );
+        assert_eq!(
+            encode_sender(&MailFrom("me@MAIL.example.com".into()), false),
+            "MAIL.example.com"
+        );
     }
 
     #[test]
