@@ -299,7 +299,9 @@ fn skip_quoted_string(input: &str) -> Option<&str> {
 
 // qcontent = qtext / quoted-pair
 fn skip_qcontent(input: &str) -> Option<&str> {
-    input.strip_prefix(is_qtext).or_else(|| skip_quoted_pair(input))
+    input
+        .strip_prefix(is_qtext)
+        .or_else(|| skip_quoted_pair(input))
 }
 
 fn is_qtext(c: char) -> bool {
@@ -330,7 +332,7 @@ fn is_vchar(c: char) -> bool {
 }
 
 /// Escapes arbitrary inputs for use as a single (not folded) word in an RFC
-/// 5322 comment. Whitespace or nested comments are not recognised but escaped
+/// 5322 comment. Whitespace and nested comments are not recognised but escaped
 /// as literal.
 pub fn escape_comment_word(s: &str) -> Cow<'_, str> {
     if s.chars().all(is_ctext) {
@@ -523,6 +525,9 @@ mod tests {
 
     #[test]
     fn decode_quoted_string_ok() {
-        assert_eq!(decode_quoted_string("\"a bc\\q🚀 \n  \\我\\\"\""), "a bcq🚀   我\"");
+        assert_eq!(
+            decode_quoted_string("\"a bc\\q🚀 \n  \\我\\\"\""),
+            "a bcq🚀   我\""
+        );
     }
 }
