@@ -10,7 +10,7 @@ use std::{
 };
 use viaspf::{SpfResult, SpfResultCause};
 
-/// A `Received-SPF` header, containing the header field content in an encoded,
+/// A `Received-SPF` header, containing the header field body in an encoded,
 /// pre-formatted form.
 #[derive(Clone, Debug, Eq, Hash, PartialEq)]
 pub struct ReceivedSpfHeader {
@@ -52,7 +52,7 @@ fn prepare_field_body(
         cause.as_ref(),
         hostname,
         ip,
-        helo_host.as_deref(),
+        helo_host,
         identity,
     );
 
@@ -263,10 +263,8 @@ where
 #[cfg(test)]
 mod tests {
     use super::*;
-    use viaspf::{
-        record::{Ip4CidrLength, Mechanism, A},
-        ErrorCause,
-    };
+    use viaspf::ErrorCause;
+    use viaspf_record::{Ip4CidrLength, Mechanism, A};
 
     #[test]
     fn format_pass_parts_ok() {
