@@ -1,3 +1,4 @@
+use async_trait::async_trait;
 use domain::{
     base::{Dname, Rtype},
     rdata::{Aaaa, Mx, Ptr, Txt, A},
@@ -6,99 +7,86 @@ use domain::{
         StubResolver,
     },
 };
-use once_cell::sync::OnceCell;
 use std::{
     error::Error,
     io::{self, ErrorKind},
     net::{IpAddr, Ipv4Addr, Ipv6Addr},
     str::FromStr,
+    sync::Arc,
     time::Duration,
 };
-use viaspf::{Lookup, LookupError, LookupResult, Name};
+use viaspf::lookup::{Lookup, LookupError, LookupResult, Name};
 
-static MOCK_RESOLVER: OnceCell<Option<MockResolver>> = OnceCell::new();
-
-/// Initialises the static mock resolver, which may be `None`.
-///
-/// Must be called exactly once during startup.
-pub fn init_mock(mock_resolver: Option<MockResolver>) {
-    // Cannot use `expect` because the mock resolver does not implement `Debug`.
-    if MOCK_RESOLVER.set(mock_resolver).is_err() {
-        panic!("mock resolver already initialized");
-    }
-}
-
-/// Returns a reference to the static mock resolver, if present.
-pub fn get_mock() -> Option<&'static MockResolver> {
-    MOCK_RESOLVER.get().expect("mock resolver not initialized").as_ref()
+pub enum Resolver {
+    Live(DomainResolver),
+    Mock(Arc<MockResolver>),
 }
 
 /// A resolver wrapping a mock `Lookup`.
 ///
 /// For use in testing.
-pub struct MockResolver(Box<dyn Lookup + Send + Sync + 'static>);
+pub struct MockResolver(Box<dyn Lookup>);
 
 impl MockResolver {
-    pub fn new(lookup: impl Lookup + Send + Sync + 'static) -> Self {
+    pub fn new(lookup: impl Lookup + 'static) -> Self {
         Self(Box::new(lookup))
     }
 }
 
+#[async_trait]
 impl Lookup for MockResolver {
-    fn lookup_a(&self, name: &Name) -> LookupResult<Vec<Ipv4Addr>> {
-        self.0.lookup_a(name)
+    async fn lookup_a(&self, name: &Name) -> LookupResult<Vec<Ipv4Addr>> {
+        self.0.lookup_a(name).await
     }
 
-    fn lookup_aaaa(&self, name: &Name) -> LookupResult<Vec<Ipv6Addr>> {
-        self.0.lookup_aaaa(name)
+    async fn lookup_aaaa(&self, name: &Name) -> LookupResult<Vec<Ipv6Addr>> {
+        self.0.lookup_aaaa(name).await
     }
 
-    fn lookup_mx(&self, name: &Name) -> LookupResult<Vec<Name>> {
-        self.0.lookup_mx(name)
+    async fn lookup_mx(&self, name: &Name) -> LookupResult<Vec<Name>> {
+        self.0.lookup_mx(name).await
     }
 
-    fn lookup_txt(&self, name: &Name) -> LookupResult<Vec<String>> {
-        self.0.lookup_txt(name)
+    async fn lookup_txt(&self, name: &Name) -> LookupResult<Vec<String>> {
+        self.0.lookup_txt(name).await
     }
 
-    fn lookup_ptr(&self, ip: IpAddr) -> LookupResult<Vec<Name>> {
-        self.0.lookup_ptr(ip)
+    async fn lookup_ptr(&self, ip: IpAddr) -> LookupResult<Vec<Name>> {
+        self.0.lookup_ptr(ip).await
     }
 }
 
 /// A resolver doing live DNS queries.
-#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
 pub struct DomainResolver {
-    timeout: Duration,
+    resolver: StubResolver,
 }
 
 impl DomainResolver {
     pub fn new(timeout: Duration) -> Self {
-        Self { timeout }
-    }
-
-    fn make_resolv_conf(&self) -> ResolvConf {
-        let mut conf = ResolvConf::default();
         let options = ResolvOptions {
-            timeout: self.timeout,
+            timeout,
             attempts: 1,
             ..Default::default()
         };
+
+        let mut conf = ResolvConf::default();
         conf.options = options;
         conf.finalize();
-        conf
+
+        let resolver = StubResolver::from_conf(conf);
+
+        Self { resolver }
     }
 }
 
+#[async_trait]
 impl Lookup for DomainResolver {
-    fn lookup_a(&self, name: &Name) -> LookupResult<Vec<Ipv4Addr>> {
+    async fn lookup_a(&self, name: &Name) -> LookupResult<Vec<Ipv4Addr>> {
         let name = to_dname(name)?;
-        let answer =
-            StubResolver::run_with_conf(self.make_resolv_conf(), |resolver| async move {
-                resolver.query((name, Rtype::A)).await
-            })
-            .map_err(to_lookup_error)?;
-        answer
+        self.resolver
+            .query((name, Rtype::A))
+            .await
+            .map_err(to_lookup_error)?
             .answer()
             .map_err(wrap_error)?
             .limit_to::<A>()
@@ -107,14 +95,12 @@ impl Lookup for DomainResolver {
             .map_err(wrap_error)
     }
 
-    fn lookup_aaaa(&self, name: &Name) -> LookupResult<Vec<Ipv6Addr>> {
+    async fn lookup_aaaa(&self, name: &Name) -> LookupResult<Vec<Ipv6Addr>> {
         let name = to_dname(name)?;
-        let answer =
-            StubResolver::run_with_conf(self.make_resolv_conf(), |resolver| async move {
-                resolver.query((name, Rtype::Aaaa)).await
-            })
-            .map_err(to_lookup_error)?;
-        answer
+        self.resolver
+            .query((name, Rtype::Aaaa))
+            .await
+            .map_err(to_lookup_error)?
             .answer()
             .map_err(wrap_error)?
             .limit_to::<Aaaa>()
@@ -123,12 +109,12 @@ impl Lookup for DomainResolver {
             .map_err(wrap_error)
     }
 
-    fn lookup_mx(&self, name: &Name) -> LookupResult<Vec<Name>> {
+    async fn lookup_mx(&self, name: &Name) -> LookupResult<Vec<Name>> {
         let name = to_dname(name)?;
-        let answer =
-            StubResolver::run_with_conf(self.make_resolv_conf(), |resolver| async move {
-                resolver.query((name, Rtype::Mx)).await
-            })
+        let answer = self
+            .resolver
+            .query((name, Rtype::Mx))
+            .await
             .map_err(to_lookup_error)?;
         let mut mxs = answer
             .answer()
@@ -144,12 +130,12 @@ impl Lookup for DomainResolver {
             .map_err(wrap_error)
     }
 
-    fn lookup_txt(&self, name: &Name) -> LookupResult<Vec<String>> {
+    async fn lookup_txt(&self, name: &Name) -> LookupResult<Vec<String>> {
         let name = to_dname(name)?;
-        let answer =
-            StubResolver::run_with_conf(self.make_resolv_conf(), |resolver| async move {
-                resolver.query((name, Rtype::Txt)).await
-            })
+        let answer = self
+            .resolver
+            .query((name, Rtype::Txt))
+            .await
             .map_err(to_lookup_error)?;
         let txts = answer
             .answer()
@@ -167,12 +153,12 @@ impl Lookup for DomainResolver {
             .map_err(wrap_error)
     }
 
-    fn lookup_ptr(&self, ip: IpAddr) -> LookupResult<Vec<Name>> {
+    async fn lookup_ptr(&self, ip: IpAddr) -> LookupResult<Vec<Name>> {
         let name = ip_to_dname(ip)?;
-        let answer =
-            StubResolver::run_with_conf(self.make_resolv_conf(), |resolver| async move {
-                resolver.query((name, Rtype::Ptr)).await
-            })
+        let answer = self
+            .resolver
+            .query((name, Rtype::Ptr))
+            .await
             .map_err(to_lookup_error)?;
         let ptrs = answer
             .answer()
@@ -229,20 +215,20 @@ mod tests {
 
     // This test is disabled because it depends on live DNS records.
     #[ignore]
-    #[test]
-    fn domain_resolver_lookup_ok() {
+    #[tokio::test]
+    async fn domain_resolver_lookup_ok() {
         let resolver = DomainResolver::new(Duration::from_secs(30));
 
         let domain = Name::new("gluet.ch").unwrap();
 
-        let ips = resolver.lookup_a(&domain);
+        let ips = resolver.lookup_a(&domain).await;
         assert!(ips.is_ok());
         let ip = ips.unwrap().into_iter().next().unwrap();
-        assert!(resolver.lookup_ptr(ip.into()).is_ok());
+        assert!(resolver.lookup_ptr(ip.into()).await.is_ok());
 
-        let ips = resolver.lookup_aaaa(&domain);
+        let ips = resolver.lookup_aaaa(&domain).await;
         assert!(ips.is_ok());
         let ip = ips.unwrap().into_iter().next().unwrap();
-        assert!(resolver.lookup_ptr(ip.into()).is_ok());
+        assert!(resolver.lookup_ptr(ip.into()).await.is_ok());
     }
 }
