@@ -1,4 +1,4 @@
-conn = mt.connect("inet:3333@localhost")
+local conn = mt.connect("inet:" .. port .. "@127.0.0.1")
 assert(conn, "could not open connection")
 
 local err = mt.macro(conn, SMFIC_CONNECT, "j", "mail.gluet.ch")
@@ -22,7 +22,7 @@ assert(err == nil, err)
 
 local err = mt.eom(conn)
 assert(err == nil, err)
-assert(mt.getreply(conn) == SMFIR_ACCEPT)
+assert(mt.getreply(conn) == SMFIR_CONTINUE)
 
 if received_spf then
   assert(mt.eom_check(conn, MT_HDRINSERT, "Received-SPF"))
