@@ -4,67 +4,72 @@ pub use common::*;
 use spf_milter::*;
 use std::{
     ffi::OsString,
-    fs,
-    net::Ipv4Addr,
+    io,
+    net::{Ipv4Addr, SocketAddr},
     path::{Path, PathBuf},
     process::ExitStatus,
-    thread,
-    time::Duration,
 };
-use viaspf::{LookupError, Name};
+use tokio::{fs, sync::mpsc};
+use viaspf::lookup::LookupError;
+
+#[tokio::test]
+async fn config_reload() {
+    let config_file = config_file_name();
 
-#[test]
-fn config_reload() {
     let opts = configure_logging(CliOptions::builder())
-        .config_file(config_file_name())
+        .config_file(&config_file)
         .build();
 
     let lookup = MockLookup::builder()
-        .lookup_txt(|name: &Name| match name.as_str() {
+        .lookup_txt(|name| match name.as_str() {
             "example.org." => Ok(vec!["v=spf1 a -all".into()]),
             _ => Err(LookupError::NoRecords),
         })
-        .lookup_a(|name: &Name| match name.as_str() {
+        .lookup_a(|name| match name.as_str() {
             "example.org." => Ok(vec![Ipv4Addr::new(123, 123, 123, 123)]),
             _ => Err(LookupError::NoRecords),
         })
         .build();
 
-    let miltertest = spawn_miltertest_runner_with_script(file!(), test_script);
-
-    // For this test the configuration is written to the file system only by the
-    // test script, which runs with a delay. Wait a bit before starting the
-    // milter.
-    thread::sleep(Duration::from_millis(200));
-
-    run_with_lookup(opts, lookup).expect("milter execution failed");
-
-    let exit_code = miltertest.join().expect("panic in miltertest runner");
-    assert!(exit_code.success(), "miltertest returned error exit code");
-}
-
-fn test_script(file_name: OsString) -> ExitStatus {
-    let config_file = config_file_name();
-
     // Write initial configuration with `Received-SPF` header.
     fs::write(
         &config_file,
         "
-        socket = inet:3333@localhost
+        socket = inet:127.0.0.1:0
         verify_helo = no
         header = Received-SPF
         ",
     )
+    .await
     .unwrap();
 
-    // Now wait as the milter needs to start and read the configuration we’ve
-    // just written.
-    thread::sleep(Duration::from_millis(200));
+    let config = Config::read_with_lookup(opts, lookup).await.unwrap();
 
-    let exit_code = run_miltertest_with_vars(&file_name, &["received_spf"]);
+    let milter = SpfMilter::spawn(config).await.unwrap();
+
+    let addr = milter.addr();
+    let reload_tx = milter.reload_handle();
+    let exit_code = run_miltertest_with_script(file!(), move |fname| {
+        Box::pin(test_script(fname, addr, reload_tx))
+    })
+    .await
+    .unwrap();
+
+    milter.shutdown().await.unwrap();
+
+    assert!(exit_code.success());
+}
+
+async fn test_script(
+    file_name: OsString,
+    addr: SocketAddr,
+    reload_config: mpsc::Sender<()>,
+) -> io::Result<ExitStatus> {
+    let config_file = config_file_name();
+
+    let exit_code = exec_miltertest_with_vars(&file_name, addr, ["received_spf"]).await?;
     if !exit_code.success() {
-        milter::shutdown();
-        return exit_code;
+        return Ok(exit_code);
     }
 
     // Write updated configuration with `Authentication-Results` header. Also
@@ -73,19 +78,18 @@ fn test_script(file_name: OsString) -> ExitStatus {
     fs::write(
         &config_file,
         "
-        socket = inet:4444@localhost
+        socket = inet:127.0.0.1:3333
         verify_helo = no
         header = Authentication-Results
         ",
     )
-    .unwrap();
+    .await?;
 
-    reload_config();
+    reload_config.send(()).await.unwrap();
 
-    let exit_code = run_miltertest(&file_name);
+    let exit_code = exec_miltertest(&file_name, addr).await?;
     if !exit_code.success() {
-        milter::shutdown();
-        return exit_code;
+        return Ok(exit_code);
     }
 
     // Write configuration with the initial `Received-SPF` header, but since the
@@ -93,21 +97,17 @@ fn test_script(file_name: OsString) -> ExitStatus {
     fs::write(
         &config_file,
         "
-        socket = inet:4444@localhost
+        socket = inet:127.0.0.1:3333
         verify_helo = no
         header = Received-SPF
         invalid_key = invalid_value
         ",
     )
-    .unwrap();
+    .await?;
 
-    reload_config();
+    reload_config.send(()).await.unwrap();
 
-    let exit_code = run_miltertest(&file_name);
-
-    milter::shutdown();
-
-    exit_code
+    exec_miltertest(&file_name, addr).await
 }
 
 fn config_file_name() -> PathBuf {
