@@ -1,4 +1,4 @@
-conn = mt.connect("inet:3333@localhost")
+local conn = mt.connect("inet:" .. port .. "@127.0.0.1")
 assert(conn, "could not open connection")
 
 local err = mt.macro(conn, SMFIC_CONNECT, "j", "mail.gluet.ch")
@@ -41,7 +41,7 @@ assert(mt.getreply(conn) == SMFIR_CONTINUE)
 
 local err = mt.eom(conn)
 assert(err == nil, err)
-assert(mt.getreply(conn) == SMFIR_ACCEPT)
+assert(mt.getreply(conn) == SMFIR_CONTINUE)
 
 assert(mt.eom_check(conn, MT_HDRDELETE, "Authentication-Results", 3));
 assert(mt.eom_check(conn, MT_HDRINSERT, "Authentication-Results",
