@@ -2,7 +2,7 @@
 -- Test 1: Accept authorised sender (HELO)
 --
 
-conn = mt.connect("inet:3333@localhost")
+local conn = mt.connect("inet:" .. port .. "@127.0.0.1")
 assert(conn, "could not open connection")
 
 local err = mt.macro(conn, SMFIC_CONNECT, "j", "mail.gluet.ch")
@@ -31,7 +31,7 @@ assert(err == nil, err)
 
 local err = mt.eom(conn)
 assert(err == nil, err)
-assert(mt.getreply(conn) == SMFIR_ACCEPT)
+assert(mt.getreply(conn) == SMFIR_CONTINUE)
 
 assert(mt.eom_check(conn, MT_HDRINSERT, "Authentication-Results",
   "mail.gluet.ch; spf=pass smtp.helo=mail.amy.org"));
@@ -43,7 +43,7 @@ assert(err == nil, err)
 -- Test 2: Accept authorised sender (MAIL FROM)
 --
 
-conn = mt.connect("inet:3333@localhost")
+local conn = mt.connect("inet:" .. port .. "@127.0.0.1")
 assert(conn, "could not open connection")
 
 local err = mt.macro(conn, SMFIC_CONNECT, "j", "mail.gluet.ch")
@@ -65,7 +65,7 @@ assert(err == nil, err)
 
 local err = mt.eom(conn)
 assert(err == nil, err)
-assert(mt.getreply(conn) == SMFIR_ACCEPT)
+assert(mt.getreply(conn) == SMFIR_CONTINUE)
 
 assert(mt.eom_check(conn, MT_HDRINSERT, "Authentication-Results",
   "mail.gluet.ch; spf=pass smtp.mailfrom=amy.org"));
@@ -77,7 +77,7 @@ assert(err == nil, err)
 -- Test 3: Reject unauthorised sender (HELO)
 --
 
-conn = mt.connect("inet:3333@localhost")
+local conn = mt.connect("inet:" .. port .. "@127.0.0.1")
 assert(conn, "could not open connection")
 
 local err = mt.macro(conn, SMFIC_CONNECT, "j", "mail.gluet.ch")
@@ -99,7 +99,7 @@ assert(err == nil, err)
 -- Test 4: Reject unauthorised sender (MAIL FROM)
 --
 
-conn = mt.connect("inet:3333@localhost")
+local conn = mt.connect("inet:" .. port .. "@127.0.0.1")
 assert(conn, "could not open connection")
 
 local err = mt.macro(conn, SMFIC_CONNECT, "j", "mail.gluet.ch")
@@ -123,7 +123,7 @@ assert(err == nil, err)
 -- Test 5: Accept exempt unauthorised sender (MAIL FROM)
 --
 
-conn = mt.connect("inet:3333@localhost")
+local conn = mt.connect("inet:" .. port .. "@127.0.0.1")
 assert(conn, "could not open connection")
 
 local err = mt.macro(conn, SMFIC_CONNECT, "j", "mail.gluet.ch")
@@ -146,7 +146,7 @@ assert(err == nil, err)
 
 local err = mt.eom(conn)
 assert(err == nil, err)
-assert(mt.getreply(conn) == SMFIR_ACCEPT)
+assert(mt.getreply(conn) == SMFIR_CONTINUE)
 
 assert(not mt.eom_check(conn, MT_HDRINSERT, "Authentication-Results"))
 
@@ -157,7 +157,7 @@ assert(err == nil, err)
 -- Test 6: Multiple messages per connection
 --
 
-conn = mt.connect("inet:3333@localhost")
+local conn = mt.connect("inet:" .. port .. "@127.0.0.1")
 assert(conn, "could not open connection")
 
 local err = mt.macro(conn, SMFIC_CONNECT, "j", "mail.gluet.ch")
@@ -180,7 +180,7 @@ assert(err == nil, err)
 
 local err = mt.eom(conn)
 assert(err == nil, err)
-assert(mt.getreply(conn) == SMFIR_ACCEPT)
+assert(mt.getreply(conn) == SMFIR_CONTINUE)
 
 assert(mt.eom_check(conn, MT_HDRINSERT, "Authentication-Results",
   "mail.gluet.ch; spf=pass smtp.mailfrom=amy.org"));
